@@ -2,12 +2,13 @@ import asyncio
 import json
 import os
 import shutil
+import warnings
 from abc import ABC, abstractmethod
 from contextlib import suppress
 from functools import partial
 from random import randint
 from tempfile import TemporaryDirectory
-from typing import Any, Callable, Optional, TypeVar
+from typing import Any, Awaitable, Callable, Optional, overload
 
 from pydoll.browser.interfaces import BrowserOptionsManager
 from pydoll.browser.managers import (
@@ -24,27 +25,25 @@ from pydoll.commands import (
     TargetCommands,
 )
 from pydoll.connection import ConnectionHandler
-from pydoll.constants import (
-    AuthChallengeResponseValues,
-    DownloadBehavior,
-    NetworkErrorReason,
-    PermissionType,
+from pydoll.exceptions import BrowserNotRunning, FailedToStartBrowser, NoValidTabFound
+from pydoll.protocol.base import Command, Response, T_CommandParams, T_CommandResponse
+from pydoll.protocol.browser.methods import (
+    GetVersionResponse,
+    GetVersionResult,
+    GetWindowForTargetResponse,
+)
+from pydoll.protocol.browser.types import Bounds, DownloadBehavior, PermissionType
+from pydoll.protocol.fetch.events import FetchEvent, RequestPausedEvent
+from pydoll.protocol.fetch.types import AuthChallengeResponseType, HeaderEntry
+from pydoll.protocol.network.types import (
+    Cookie,
+    CookieParam,
+    ErrorReason,
     RequestMethod,
     ResourceType,
 )
-from pydoll.exceptions import BrowserNotRunning, FailedToStartBrowser, NoValidTabFound
-from pydoll.protocol.base import Command, Response
-from pydoll.protocol.browser.responses import (
-    GetVersionResponse,
-    GetVersionResultDict,
-    GetWindowForTargetResponse,
-)
-from pydoll.protocol.browser.types import WindowBoundsDict
-from pydoll.protocol.fetch.events import FetchEvent
-from pydoll.protocol.fetch.types import HeaderEntry
-from pydoll.protocol.network.types import Cookie, CookieParam, RequestPausedEvent
-from pydoll.protocol.storage.responses import GetCookiesResponse
-from pydoll.protocol.target.responses import (
+from pydoll.protocol.storage.methods import GetCookiesResponse
+from pydoll.protocol.target.methods import (
     CreateBrowserContextResponse,
     CreateTargetResponse,
     GetBrowserContextsResponse,
@@ -52,8 +51,6 @@ from pydoll.protocol.target.responses import (
 )
 from pydoll.protocol.target.types import TargetInfo
 
-T = TypeVar('T')
-
 
 class Browser(ABC):  # noqa: PLR0904
     """
@@ -110,7 +107,7 @@ class Browser(ABC):  # noqa: PLR0904
         Start browser process and establish CDP connection.
 
         Args:
-            headless: Run without UI.
+            headless: Deprecated. Use `options.headless = True` instead.
 
         Returns:
             Initial tab for interaction.
@@ -118,12 +115,16 @@ class Browser(ABC):  # noqa: PLR0904
         Raises:
             FailedToStartBrowser: If the browser fails to start or connect.
         """
-        binary_location = self.options.binary_location or self._get_default_binary_location()
-
         if headless:
-            headless_arg = '--headless'
-            if headless_arg not in self.options.arguments:
-                self.options.add_argument(headless_arg)
+            warnings.warn(
+                "The 'headless' parameter is deprecated and will be removed in a future version. "
+                'Use `options.headless = True` instead.',
+                DeprecationWarning,
+                stacklevel=2,
+            )
+            self.options.headless = headless
+
+        binary_location = self.options.binary_location or self._get_default_binary_location()
 
         self._setup_user_dir()
         proxy_config = self._proxy_manager.get_proxy_credentials()
@@ -218,7 +219,8 @@ class Browser(ABC):  # noqa: PLR0904
         )
         target_id = response['result']['targetId']
         tab = Tab(self, self._connection_port, target_id, browser_context_id)
-        if url: await tab.go_to(url)
+        if url:
+            await tab.go_to(url)
         return tab
 
     async def get_targets(self) -> list[TargetInfo]:
@@ -255,7 +257,11 @@ class Browser(ABC):  # noqa: PLR0904
     async def set_download_path(self, path: str, browser_context_id: Optional[str] = None):
         """Set download directory path (convenience method for set_download_behavior)."""
         return await self._execute_command(
-            BrowserCommands.set_download_behavior(DownloadBehavior.ALLOW, path, browser_context_id)
+            BrowserCommands.set_download_behavior(
+                behavior=DownloadBehavior.ALLOW,
+                download_path=path,
+                browser_context_id=browser_context_id,
+            )
         )
 
     async def set_download_behavior(
@@ -276,7 +282,10 @@ class Browser(ABC):  # noqa: PLR0904
         """
         return await self._execute_command(
             BrowserCommands.set_download_behavior(
-                behavior, download_path, browser_context_id, events_enabled
+                behavior=behavior,
+                download_path=download_path,
+                browser_context_id=browser_context_id,
+                events_enabled=events_enabled,
             )
         )
 
@@ -297,7 +306,7 @@ class Browser(ABC):  # noqa: PLR0904
         )
         return response['result']['cookies']
 
-    async def get_version(self) -> GetVersionResultDict:
+    async def get_version(self) -> GetVersionResult:
         """Get browser version and CDP protocol information."""
         response: GetVersionResponse = await self._execute_command(BrowserCommands.get_version())
         return response['result']
@@ -334,7 +343,7 @@ class Browser(ABC):  # noqa: PLR0904
         window_id = await self.get_window_id()
         return await self._execute_command(BrowserCommands.set_window_minimized(window_id))
 
-    async def set_window_bounds(self, bounds: WindowBoundsDict):
+    async def set_window_bounds(self, bounds: Bounds):
         """
         Set window position and/or size.
 
@@ -369,9 +378,15 @@ class Browser(ABC):  # noqa: PLR0904
         """Reset all permissions to defaults and restore prompting behavior."""
         return await self._execute_command(BrowserCommands.reset_permissions(browser_context_id))
 
+    @overload
     async def on(
         self, event_name: str, callback: Callable[[Any], Any], temporary: bool = False
-    ) -> int:
+    ) -> int: ...
+    @overload
+    async def on(
+        self, event_name: str, callback: Callable[[Any], Awaitable[Any]], temporary: bool = False
+    ) -> int: ...
+    async def on(self, event_name, callback, temporary: bool = False) -> int:
         """
         Register CDP event listener at browser level.
 
@@ -400,6 +415,10 @@ class Browser(ABC):  # noqa: PLR0904
             event_name, function_to_register, temporary
         )
 
+    async def remove_callback(self, callback_id: int):
+        """Remove callback from browser."""
+        return await self._connection_handler.remove_callback(callback_id)
+
     async def enable_fetch_events(
         self,
         handle_auth_requests: bool = False,
@@ -437,7 +456,7 @@ class Browser(ABC):  # noqa: PLR0904
         """Disable runtime events."""
         return await self._connection_handler.execute_command(RuntimeCommands.disable())
 
-    async def continue_request(  # noqa: PLR0913, PLR0917
+    async def continue_request(
         self,
         request_id: str,
         url: Optional[str] = None,
@@ -460,7 +479,7 @@ class Browser(ABC):  # noqa: PLR0904
             )
         )
 
-    async def fail_request(self, request_id: str, error_reason: NetworkErrorReason):
+    async def fail_request(self, request_id: str, error_reason: ErrorReason):
         """Fail request with error code."""
         return await self._execute_command(FetchCommands.fail_request(request_id, error_reason))
 
@@ -505,7 +524,7 @@ class Browser(ABC):  # noqa: PLR0904
         response: Response = await self._execute_command(
             FetchCommands.continue_request_with_auth(
                 request_id,
-                auth_challenge_response=AuthChallengeResponseValues.PROVIDE_CREDENTIALS,
+                auth_challenge_response=AuthChallengeResponseType.PROVIDE_CREDENTIALS,
                 proxy_username=proxy_username,
                 proxy_password=proxy_password,
             )
@@ -584,7 +603,9 @@ class Browser(ABC):  # noqa: PLR0904
 
         return False
 
-    async def _execute_command(self, command: Command[T], timeout: int = 10) -> T:
+    async def _execute_command(
+        self, command: Command[T_CommandParams, T_CommandResponse], timeout: int = 10
+    ) -> T_CommandResponse:
         """Execute CDP command and return result (core method for browser communication)."""
         return await self._connection_handler.execute_command(command, timeout=timeout)
 
