@@ -1,17 +1,20 @@
 import asyncio
-from typing import TYPE_CHECKING, Literal, Optional, TypeVar, Union, overload
+from typing import TYPE_CHECKING, Literal, Optional, Union, overload
 
 from pydoll.commands import (
     DomCommands,
     RuntimeCommands,
 )
+from pydoll.connection.connection_handler import ConnectionHandler
 from pydoll.constants import By, Scripts
 from pydoll.exceptions import ElementNotFound, WaitElementTimeout
-from pydoll.protocol.base import Command
-from pydoll.protocol.dom.responses import DescribeNodeResponse
+from pydoll.protocol.base import Command, T_CommandParams, T_CommandResponse
+from pydoll.protocol.dom.methods import DescribeNodeResponse
 from pydoll.protocol.dom.types import Node
-from pydoll.protocol.runtime.responses import (
+from pydoll.protocol.runtime.methods import (
+    CallFunctionOnParams,
     CallFunctionOnResponse,
+    EvaluateParams,
     EvaluateResponse,
     GetPropertiesResponse,
 )
@@ -19,8 +22,6 @@ from pydoll.protocol.runtime.responses import (
 if TYPE_CHECKING:
     from pydoll.elements.web_element import WebElement
 
-T = TypeVar('T')
-
 
 def create_web_element(*args, **kwargs):
     """
@@ -44,6 +45,9 @@ class FindElementsMixin:
     complex location logic themselves.
     """
 
+    if TYPE_CHECKING:
+        _connection_handler: ConnectionHandler
+
     @overload
     async def find(
         self,
@@ -114,7 +118,7 @@ class FindElementsMixin:
         **attributes: dict[str, str],
     ) -> Union['WebElement', list['WebElement'], None]: ...
 
-    async def find(  # noqa: PLR0913, PLR0917
+    async def find(
         self,
         id: Optional[str] = None,
         class_name: Optional[str] = None,
@@ -327,7 +331,7 @@ class FindElementsMixin:
 
         object_id = response_for_command['result']['result']['objectId']
         attributes = await self._get_object_attributes(object_id=object_id)
-        return create_web_element(object_id, self._connection_handler, by, value, attributes)  # type: ignore
+        return create_web_element(object_id, self._connection_handler, by, value, attributes)
 
     async def _find_elements(
         self, by: By, value: str, raise_exc: bool = True
@@ -368,7 +372,7 @@ class FindElementsMixin:
         query_response: GetPropertiesResponse = await self._execute_command(
             RuntimeCommands.get_properties(object_id=object_id)
         )
-        response = []
+        response: list[str] = []
         for query in query_response['result']['result']:
             query_value = query.get('value', {})
             if query_value and query_value['type'] == 'object':
@@ -386,7 +390,7 @@ class FindElementsMixin:
             attributes.extend(['tag_name', tag_name])
 
             elements.append(
-                create_web_element(object_id, self._connection_handler, by, value, attributes)  # type: ignore
+                create_web_element(object_id, self._connection_handler, by, value, attributes)
             )
         return elements
 
@@ -400,7 +404,7 @@ class FindElementsMixin:
         attributes.extend(['tag_name', tag_name])
         return attributes
 
-    def _get_by_and_value(  # noqa: PLR0913, PLR0917
+    def _get_by_and_value(
         self,
         by_map: dict[str, By],
         id: Optional[str] = None,
@@ -490,9 +494,11 @@ class FindElementsMixin:
         )
         return response['result']['node']
 
-    async def _execute_command(self, command: Command[T]) -> T:
+    async def _execute_command(
+        self, command: Command[T_CommandParams, T_CommandResponse]
+    ) -> T_CommandResponse:
         """Execute CDP command via connection handler (60s timeout)."""
-        return await self._connection_handler.execute_command(command, timeout=60)  # type: ignore
+        return await self._connection_handler.execute_command(command, timeout=60)
 
     def _get_find_element_command(self, by: By, value: str, object_id: str = ''):
         """
@@ -505,6 +511,10 @@ class FindElementsMixin:
         - NAME: converts to XPath expression
         """
         escaped_value = value.replace('"', '\\"')
+        command: Union[
+            Command[CallFunctionOnParams, CallFunctionOnResponse],
+            Command[EvaluateParams, EvaluateResponse],
+        ]
         match by:
             case By.CLASS_NAME:
                 selector = f'.{escaped_value}'
@@ -539,6 +549,10 @@ class FindElementsMixin:
         Handles same special cases and selector type conversions.
         """
         escaped_value = value.replace('"', '\\"')
+        command: Union[
+            Command[CallFunctionOnParams, CallFunctionOnResponse],
+            Command[EvaluateParams, EvaluateResponse],
+        ]
         match by:
             case By.CLASS_NAME:
                 selector = f'.{escaped_value}'
@@ -568,6 +582,10 @@ class FindElementsMixin:
         XPath requires special handling vs CSS selectors. Ensures relative
         XPath for context-based searches.
         """
+        command: Union[
+            Command[CallFunctionOnParams, CallFunctionOnResponse],
+            Command[EvaluateParams, EvaluateResponse],
+        ]
         escaped_value = xpath.replace('"', '\\"')
         if object_id:
             escaped_value = self._ensure_relative_xpath(escaped_value)
@@ -590,6 +608,10 @@ class FindElementsMixin:
         XPath for context-based searches.
         """
         escaped_value = xpath.replace('"', '\\"')
+        command: Union[
+            Command[CallFunctionOnParams, CallFunctionOnResponse],
+            Command[EvaluateParams, EvaluateResponse],
+        ]
         if object_id:
             escaped_value = self._ensure_relative_xpath(escaped_value)
             script = Scripts.FIND_RELATIVE_XPATH_ELEMENTS.replace('{escaped_value}', escaped_value)
